
var subscriptionTree;
var feedService;
var contentBrowser;
var promptService;
var dragService;
var scrollTimer;
var windowManager;

var feedServiceListener = {
	onChanged: function() {
		dump("newsSidebar: rebuilding tree...\n");
		var selectionIndex = subscriptionTree.view.selection.currentIndex;
		subscriptionTree.builder.rebuild();
		if (selectionIndex != -1) {
			subscriptionTree.view.selection.select(selectionIndex);
		}
		updateSelection();
	},
	
	onNewCountUpdate: function() {
	}
}

var newsDNDObserver = {

    onDragOver: function(aEvent, aFlavor, aSession) {
        dump("newsDNDObserver: onDragOver\n");
    },
    
    onDragEnter: function(aEvent, aSession) {
        dump("newsDNDObserver: onDragEnter\n");
    },
    
    onDragExit: function(aEvent, aSession) {
        dump("newsDNDObserver: onDragExit\n");
    },

	onDragStart: function(event, transferData, action) {
		var index = subscriptionTree.view.selection.currentIndex;
		if(index != -1) {
			var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
			var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
			var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
		} else {
			return;
		}
		transferData.data = new TransferData();
		transferData.data.addDataForFlavour("moz/rdfitem", id);
	},
	
	onDrop: function(aEvent, aDropData, aSession) {
		dump("newsDNDObserver: onDrop: contentType: " + aDropData.flavour.contentType + " data: " + aDropData.data + "\n");
		switch(aDropData.flavour.contentType) {
			case "application/x-moz-file":
				var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
				var fileHandler = ios.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
				var url = fileHandler.getURLSpecFromFile(aDropData.data);
				break;
			case "text/x-moz-url":
				var url = aDropData.data.split("\n")[0];
				break;
		}
		var feedService = Components.classes["@flock.com/feed-service;1"].getService(Components.interfaces.flockIFeedService);
		feedService.subscribeFeed(url, null, null, false);
	},
	
	getSupportedFlavours: function() {
		var flavors = new FlavourSet();
		flavors.appendFlavour("application/x-moz-file", "nsIFile");
		flavors.appendFlavour("text/x-moz-url");
		return flavors;
	}
	
}

var treeBuilderObserver = {
	canDrop: function(index, orient) {
		dump("treeBuilderObserver: canDrop: index: " + index + " orient: " + orient + "\n");
		if (index == -1) return false;
		var dragSession = dragService.getCurrentSession();
		var target_type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
		if (dragSession.isDataFlavorSupported("moz/rdfitem")) {
			var source_type = subscriptionTree.view.getCellText(subscriptionTree.view.selection.currentIndex, subscriptionTree.columns.getNamedColumn("type"));
			if (source_type == "root" || target_type == "root" || target_type == "flagged_collection") return false;
			if (source_type == "collection" && orient == 0) return false;
			return true;
		}
		if (dragSession.isDataFlavorSupported("text/x-moz-url")) {
			if (target_type == "root" || target_type == "flagged_collection") return false;
			return true;
		}
		return false;
	},
	
	onDrop: function(index, orient) {
		var dragSession = dragService.getCurrentSession();

		if (dragSession.isDataFlavorSupported("moz/rdfitem")) {
			var trans = nsTransferable.createTransferable();
			trans.addDataFlavor("moz/rdfitem");
			dragSession.getData(trans, 0);
			var data = { };
			var length = { };
			trans.getTransferData("moz/rdfitem", data, length);
			data.value.QueryInterface(Components.interfaces.nsISupportsString);
			//var source = data.value.toString();
	
			var source = subscriptionTree.builder.getResourceAtIndex(subscriptionTree.view.selection.currentIndex).ValueUTF8;
			var sourceType = subscriptionTree.view.getCellText(subscriptionTree.view.selection.currentIndex, subscriptionTree.columns.getNamedColumn("type"));
			var target = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
			
			var isContainerTarget = subscriptionTree.view.isContainer(index);
			var isContainerTargetExpanded = subscriptionTree.view.isContainerOpen(index);
			var isContainerTargetEmpty = subscriptionTree.view.isContainerEmpty(index);
			
			if (isContainerTarget && isContainerTargetExpanded && !isContainerTargetEmpty && orient == 1) {
				target = subscriptionTree.builder.getResourceAtIndex(index + 1).ValueUTF8;
				orient = -1;
			}
			
			switch (sourceType) {
				case "feed":
					feedService.moveFeed(source, target, orient);
					break;
				case "collection":
					feedService.moveCollection(source, target, orient);
					break;
			}
			
			dump("treeBuilderObserver: onDrop: source: " + source + " target: " + target + " sourceType: " + sourceType + " orient: " + orient + "\n");
		}
		
		if (dragSession.isDataFlavorSupported("text/x-moz-url")) {
			var trans = nsTransferable.createTransferable();
			trans.addDataFlavor("text/x-moz-url");
			dragSession.getData(trans, 0);
			var data = { };
			var length = { };
			trans.getTransferData("text/x-moz-url", data, length);
			data.value.QueryInterface(Components.interfaces.nsISupportsString);
			var source = data.value.toString().split("\n")[0];
			
			var target = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
			
			var isContainerTarget = subscriptionTree.view.isContainer(index);
			var isContainerTargetExpanded = subscriptionTree.view.isContainerOpen(index);
			var isContainerTargetEmpty = subscriptionTree.view.isContainerEmpty(index);
			
			if (isContainerTarget && isContainerTargetExpanded && !isContainerTargetEmpty && orient == 1) {
				target = subscriptionTree.builder.getResourceAtIndex(index + 1).ValueUTF8;
				orient = -1;
			}
			
			feedService.subscribeFeed(source, target, orient, false);
			
			dump("treeBuilderObserver: onDrop: source: " + source + " target: " + target + " isContainerTarget: " + isContainerTarget + " isContainerTargetExpanded: " + isContainerTargetExpanded + " isContainerTargetEmpty: " + isContainerTargetEmpty + "\n");
		}
	},
	
	onToggleOpenState: function() {
	}
};

function init() {
	subscriptionTree = document.getElementById("subscriptionTree");
	
	feedService = Components.classes["@flock.com/feed-service;1"].getService(Components.interfaces.flockIFeedService);
	promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
	dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
	scrollTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
	windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	
	var recentWindow = windowManager.getMostRecentWindow("navigator:browser");
	contentBrowser = recentWindow.document.getElementById('content');
	
	subscriptionTree.database.AddDataSource(feedService.getSubscriptions());
	subscriptionTree.builder.rebuild();
	subscriptionTree.builder.addObserver(treeBuilderObserver);
	
	feedService.addObserver(feedServiceListener);
	feedService.resetNewCount();
	
	if (!subscriptionTree.view.isContainerOpen(0)) {
		for (var c = 0; c < subscriptionTree.view.rowCount; c++) {
			if (subscriptionTree.view.isContainer(c) && !subscriptionTree.view.isContainerOpen(c)) {
				subscriptionTree.view.toggleOpenState(c);
			}
		}
	}

	if(contentBrowser.currentURI.spec == "about:blank") {
		subscriptionTree.view.selection.select(0);
		renderContent();
	}

	updateSelection();	
}

function updateSelection() {
	if(contentBrowser.currentURI.scheme == "feed" || contentBrowser.currentURI.scheme == "feeds") {
		var feedUrl = decodeURIComponent(contentBrowser.currentURI.spec.split(contentBrowser.currentURI.scheme + ":")[1]);
		dump("feedUrl: " + feedUrl + "\n");
		for (var c = 0; c < subscriptionTree.view.rowCount; c++) {
			switch (subscriptionTree.view.getCellText(c, subscriptionTree.columns.getNamedColumn("type"))) {
				case "feed":
					if (subscriptionTree.builder.getResourceAtIndex(c).ValueUTF8 == feedUrl) {
						subscriptionTree.view.selection.select(c);
					}
					break;
				case "root":
					if (feedUrl == "") {
						subscriptionTree.view.selection.select(c);
					}
					break;
				case "collection":
					if (feedUrl == subscriptionTree.view.getCellText(c, subscriptionTree.columns.getNamedColumn("title"))) {
						subscriptionTree.view.selection.select(c);
					}
					break;
				case "flagged_collection":
					if (feedUrl == "saved") {
						subscriptionTree.view.selection.select(c);
					}
					break;
			}
		}
	}
}

function close() {
	feedService.removeObserver(feedServiceListener);
	feedService.resetNewCount();
}

function doOnSelect() {
	//renderContent();
}

function doOnClick(event) {
	dump("News: onClick\n");
	var obj = {};
	var row = {};
	subscriptionTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, {}, obj);
	row = row.value;
	if(event.button == 2 || event.originalTarget.localName != "treechildren" || row == -1) {
		return;
	}
	if(event.ctrlKey || event.metaKey) {
		var newTab = contentBrowser.addTab();
		contentBrowser.selectedTab = newTab;
	}
	if(obj.value == "twisty") return;
	renderContent();
}

function doOnKeyPress(event) {
	dump("News: onKeyPress: " + event.keyCode + "\n");
	if (event.keyCode == 38 || event.keyCode == 40) {
		var timerCallback = {
			notify: function(aTimer) { renderContent(); }
		}
		scrollTimer.cancel();
		scrollTimer.initWithCallback(timerCallback, 400, scrollTimer.TYPE_ONE_SHOT);
	}
}

function renderContent() {
	var index = subscriptionTree.view.selection.currentIndex;
	if(index != -1) {
		var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
		var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
		var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
	} else {
		return;
	}
	
	dump("News: index: " + index + " type: " + type + " id: " + id + "\n");
	
	switch (type) {
		case "feed":
			contentBrowser.loadURI("feeds:" + encodeURIComponent(id));
			break;
		case "root":
			contentBrowser.loadURI("feeds:");
			break;
		case "collection":
			contentBrowser.loadURI("feeds:" + encodeURIComponent(title));
			break;
		case "flagged_collection":
			contentBrowser.loadURI("feeds:saved");
			break;
	}
	
	var prefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);
	if (prefs.getBoolPref("flock.feeds.auto_mark_as_read", true)) {
		if (type != "root") {
			function latentMark() {
				feedService.markAllRead(id);
				subscriptionTree.view.selection.select(index);
			}
			setTimeout(latentMark, 500);
		}
	}
}

function newCollection(event) {
	var check = {value: false};
	var input = {value: ""};
	var result = promptService.prompt(null, "New Collection", "Collection Name:", input, null, check);
	if (result) {
		feedService.addCollection(input.value);
	}
}

function editItem(event) {
	var index = subscriptionTree.view.selection.currentIndex;
	if(index != -1) {
		var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
		var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
		var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
	} else {
		return;
	}

	var check = {value: false};
	var input = {value: title};
	switch (type) {
		case "feed":
			var result = promptService.prompt(null, "Edit Feed", "Feed Title:", input, null, check);
			if (result) {
				feedService.renameItem(id, input.value);
			}
			break;
		case "collection":
			var result = promptService.prompt(null, "Edit Section", "Section Title:", input, null, check);
			if (result) {
				feedService.renameItem(id, input.value);
			}
			break;
		case "root":
			break;
	}
}

function deleteItem(event) {
	var index = subscriptionTree.view.selection.currentIndex;
	if(index != -1) {
		var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
		var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
		var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
	} else {
		return;
	}

	switch (type) {
		case "feed":
			feedService.unsubscribeFeed(id);
			break;
		case "collection":
			feedService.removeCollection(id);
			break;
		case "root":
			break;
	}
}

function markAsRead(event) {
	var index = subscriptionTree.view.selection.currentIndex;
	if(index != -1) {
		var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
		var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
		var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
	} else {
		return;
	}

	feedService.markAllRead(id);
}

function markAsUnread(event) {
	var index = subscriptionTree.view.selection.currentIndex;
	if(index != -1) {
		var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
		var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
		var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
	} else {
		return;
	}

	feedService.markAllUnread(id);
}

importSourcePack = function() {
    window.openDialog("chrome://browser/content/flock/feed/importOPML.xul", "ImportOPML", "modal,centerscreen");
}

manuallyAddSource = function() {
    window.openDialog("chrome://browser/content/flock/feed/addFeedUrl.xul", "AddFeedUrl", "modal,centerscreen");
}
